﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.Drawing.Printing;
using Leadtools;
using Leadtools.Codecs;
using Leadtools.WinForms;


namespace PrintImageTutorial
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        // 加载图像时使用的 RasterCodecs 对象
        private RasterCodecs _rasterCodecs;
        // 整个演示中我们会用到的 PrintDocument 对象
        private PrintDocument _printDocument;
        // 当前要打印的页码
        private int _currentPrintPageNumber;
        // 当前的图像文件名称
        private string _currentImageFileName;


        protected override void OnLoad(EventArgs e)
        {
            // 在加载图像时初始化 RasterCodecs 对象
            this._rasterCodecs = new RasterCodecs();

            // 检查是否安装打印机
            if (PrinterSettings.InstalledPrinters == null || PrinterSettings.InstalledPrinters.Count < 1)
            {
                MessageBox.Show(
                   this,
                   "此机器上未安装打印机，此程序无法继续运行",
                   "PrintImageTutorial",
                   MessageBoxButtons.OK,
                   MessageBoxIcon.Exclamation);
                this.Close();
            }
            else
            {
                // 创建我们将要使用的print document对象
                this._printDocument = new PrintDocument();
                // 为打印事件添加句柄
                this._printDocument.BeginPrint += new PrintEventHandler(_printDocument_BeginPrint);
                this._printDocument.PrintPage += new PrintPageEventHandler(_printDocument_PrintPage);
                this._printDocument.EndPrint += new PrintEventHandler(_printDocument_EndPrint);
            }

            base.OnLoad(e);
        }

        protected override void OnFormClosed(FormClosedEventArgs e)
        {
            // 释放我们使用的资源

            if (this._printDocument != null)
            {
                this._printDocument.Dispose();
            }

            if (this._rasterCodecs != null)
            {
                this._rasterCodecs.Dispose();
            }

            base.OnFormClosed(e);
        }

        private void fileToolStripMenuItem_DropDownOpening(object sender, EventArgs e)
        {
            // 更新UI状态
            printSetupToolStripMenuItem.Enabled = (rasterImageViewer1.Image != null);
            printToolStripMenuItem.Enabled = (rasterImageViewer1.Image != null);
        }

        private void openToolStripMenuItem_Click(object sender, EventArgs e)
        {
            // 将图像加载至查看器
            using (OpenFileDialog dlg = new OpenFileDialog())
            {
                dlg.Filter = "All Files|*.*";
                if (dlg.ShowDialog(this) == DialogResult.OK)
                {
                    try
                    {
                        // 加载文件中的所有页面
                        rasterImageViewer1.Image = this._rasterCodecs.Load(dlg.FileName);
                        Controls.Add(rasterImageViewer1);
                        rasterImageViewer1.BringToFront();
                        this._currentImageFileName = dlg.FileName;
                        UpdateCaption();
                    }
                    catch (Exception ex)
                    {
                        MessageBox.Show(this, ex.Message, "打印图像演示", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    }
                }
            }
        }
        private void printPreviewToolStripMenuItem_Click(object sender, EventArgs e)
        {
            // 设置打印文档
            SetupPrintDocument();

            // 使用 .NET PrintPreviewDialog
            using (PrintPreviewDialog dlg = new PrintPreviewDialog())
            {
                // 显示对话框
                dlg.Document = this._printDocument;
                dlg.WindowState = FormWindowState.Maximized;
                dlg.ShowDialog(this);
            }

        }


        private void printSetupToolStripMenuItem_Click(object sender, EventArgs e)
        {
            // 使用 .NET PageSetupDialog
            using (PageSetupDialog dlg = new PageSetupDialog())
            {
                dlg.Document = this._printDocument;
                dlg.ShowDialog(this);
            }
        }


        private void printToolStripMenuItem_Click(object sender, EventArgs e)
        {
            // 用户可能会使用快捷键，因此即使我们禁用了菜单项，也需要检查打印状态是否可用
            if (rasterImageViewer1.Image == null)
            {
                return;
            }

            // 显示打印预览对话框，然后直接打印

            // 设置打印文档
            SetupPrintDocument();

            this._printDocument.Print();
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            // 关闭应用程序
            Close();
        }

        private void firstPageToolStripMenuItem_Click(object sender, EventArgs e)
        {
            // 转到图像第一页
            rasterImageViewer1.Image.Page = 1;
            UpdateCaption();
        }

        private void pageToolStripMenuItem_DropDownOpening(object sender, EventArgs e)
        {
            //更新UI状态
            firstPageToolStripMenuItem.Enabled = (rasterImageViewer1.Image != null && rasterImageViewer1.Image.Page > 1);
            previousPageToolStripMenuItem.Enabled = firstPageToolStripMenuItem.Enabled;
            nextPageToolStripMenuItem.Enabled = (rasterImageViewer1.Image != null && rasterImageViewer1.Image.Page < rasterImageViewer1.Image.PageCount);
            lastPageToolStripMenuItem.Enabled = nextPageToolStripMenuItem.Enabled;
        }

        private void previousPageToolStripMenuItem_Click(object sender, EventArgs e)
        {
            // 转到图像的上一页
            rasterImageViewer1.Image.Page--;
            UpdateCaption();
        }

        private void nextPageToolStripMenuItem_Click(object sender, EventArgs e)
        {
            // 转到图像的下一页
            rasterImageViewer1.Image.Page++;
            UpdateCaption();
        }

        private void lastPageToolStripMenuItem_Click(object sender, EventArgs e)
        {
            // 转到图像的尾页
            rasterImageViewer1.Image.Page = rasterImageViewer1.Image.PageCount;
            UpdateCaption();
        }

        private void usePageMarginsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            // 切换选项
            usePageMarginsToolStripMenuItem.Checked = !usePageMarginsToolStripMenuItem.Checked;
        }

        private void fitImageToPageToolStripMenuItem_Click(object sender, EventArgs e)
        {
            // 切换选项
            fitImageToPageToolStripMenuItem.Checked = !fitImageToPageToolStripMenuItem.Checked;
        }


        private void UpdateCaption()
        {
            // 更新演示的标题，显示加载的图像文件名字和当前页码
            if (rasterImageViewer1.Image != null)
            {
                Text = string.Format(
                   "{0} - 第 {1} 页 (共 {2} 页) - 打印图像演示",
                   this._currentImageFileName,
                   rasterImageViewer1.Image.Page,
                   rasterImageViewer1.Image.PageCount);
            }
            else
            {
                Text = "打印图像演示";
            }
        }

        private void SetupPrintDocument()
        {
            // 在打印预览前或打印前调用，用于设置文档

            // 最小值/最大值 为图像的页面数
            this._printDocument.PrinterSettings.MinimumPage = 1;
            this._printDocument.PrinterSettings.MaximumPage = rasterImageViewer1.Image.PageCount;

            // 默认打印所有页面
            this._printDocument.PrinterSettings.FromPage = this._printDocument.PrinterSettings.MinimumPage;
            this._printDocument.PrinterSettings.ToPage = this._printDocument.PrinterSettings.MaximumPage;

            // 设置文档名字
            this._printDocument.DocumentName = this._currentImageFileName;
        }

        private void _printDocument_BeginPrint(object sender, PrintEventArgs e)
        {
            // 重置页码为首页
            this._currentPrintPageNumber = 1;
        }

        private void _printDocument_PrintPage(object sender, PrintPageEventArgs e)
        {
            // 打印一页

            // 获取 print document 对象
            PrintDocument document = sender as PrintDocument;

            // 创建一个新的LEADTOOLS image printer类
            RasterImagePrinter printer = new RasterImagePrinter();

            // 设置 document 对象以便进行页面计算
            printer.PrintDocument = document;

            // 检查我们是否想要使图像适应
            if (fitImageToPageToolStripMenuItem.Checked)
            {
                // 若为Yes, 将图像调整到最大打印区域并居中
                printer.SizeMode = RasterPaintSizeMode.FitAlways;
                printer.HorizontalAlignMode = RasterPaintAlignMode.Center;
                printer.VerticalAlignMode = RasterPaintAlignMode.Center;
            }
            else
            {
                // 若为No, 正常打印 (原始大小)
                printer.SizeMode = RasterPaintSizeMode.Normal;
                printer.HorizontalAlignMode = RasterPaintAlignMode.Near;
                printer.VerticalAlignMode = RasterPaintAlignMode.Near;
            }

            // 考虑具有不同水平和垂直分辨率的传真图像
            printer.UseDpi = true;

            // 打印整个图像
            printer.ImageRectangle = Rectangle.Empty;

            // 使用最大页面维度，这和使用Windows照片库打印等效
            printer.PageRectangle = RectangleF.Empty;
            // 无论我们是否要忽略页边距，都会通知打印机
            printer.UseMargins = usePageMarginsToolStripMenuItem.Checked;

            // 打印当前页
            printer.Print(rasterImageViewer1.Image, this._currentPrintPageNumber, e);

            // 转到下一页
            this._currentPrintPageNumber++;

            // 无论我们是否要打印更多的页面，都通知打印机
            if (this._currentPrintPageNumber <= document.PrinterSettings.ToPage)
            {
                e.HasMorePages = true;
            }
            else
            {
                e.HasMorePages = false;
            }
        }

        private void _printDocument_EndPrint(object sender, PrintEventArgs e)
        {
            //此教程中此处不执行任何操作
        }


    }
}
